#!/bin/sh
help="$(cat<<EOF
Generate snapshot tarball from git repository.

Syntax:
  sh $0 [<options>]
    Generate a snapshot tarball

Options:
  --update-config
    Update spec file, README.foo, etc. with the correct timestamps,
    commit ids, etc.
  --skip-pull
    Do not fetch updates from upstream repository.
  --no-checks
    Disable check runs.

Requires:
  git, automake, autoconf, libtool
EOF
)"

GIT="${GIT-git}"
RPM="${RPM-rpm}"
SED="${SED-sed}"

# Local settings
name="xorg-x11-drv-radeonhd"
tarbase="xf86-video-radeonhd"
builddir="${tarbase}.build"
srcdir="${tarbase}.src"
specfile="${name}.spec"
date="${date-$(date +%Y%m%d)}"

do_branch="false"
branch="fedora/snapshot"

# Check files and directories before starting to do anything
if [ -d "${builddir}" ]; then
    echo "Remove the \"${builddir}\" directory first !"
    exit 1
fi

# Default settings
maintainer_name="Main Tainer"
maintainer_mail="main@tainer.invalid"

opt_update_config="false"
opt_skip_pull="false"
opt_no_checks="false"

# Pull in settings, overriding the defaults
if test -f "$(dirname "$0")/.snapshot.cfg"; then
    . "$(dirname "$0")/.snapshot.cfg"
fi

# Parse command line options
for opt
do
    case "$opt" in
	--skip-pull)      opt_skip_pull=":" ;;
	--update-config)  opt_update_config=":" ;;
	--no-checks)      opt_no_checks=":" ;;
	-h|--help)
	    echo "$help"
	    exit 0
	    ;;
	*)
	    echo "FATAL: Unhandled command line option: $opt"
	    exit 13
	    ;;
    esac
done


# Syntax: sed_update_cond <FILENAME> [<sed-options>]
sed_update_cond() {
    old="$1"
    new="$1.new"
    shift
    if $SED "$@" < "${old}" > "${new}"; then
	if cmp "${new}" "${old}"; then :
	else
	    mv -f "${new}" "${old}"
	    return
	fi
    fi
    rm -f "${new}"
}


# Syntax: update_changelog <SPECFILE> <old_commit_id> <new_commit_id>
update_changelog() {
    specfile="$1"
    old_commit="$2"
    new_commit="$3"

    rpm_date=`date "+%a %b %d %Y"`

    fullspecfile="$PWD/$specfile"
    [ -f "$specfile" ]
    rpm_version="$($RPM --queryformat "%{VERSION}-%{RELEASE}\n" -q --specfile "${specfile}" | $SED -n 1p | $SED 's/\.fc[0-9][0-9]*$//')"

    pre_update_dir=`pwd`
    cd "$(dirname "$0")/xf86-video-radeonhd.src"
    [ -d ".git" ]

    {
        sed '/^%changelog$/ q' "${fullspecfile}"
        echo "* ${rpm_date} ${maintainer_name} <${maintainer_mail}> - ${rpm_version}"
        echo "- New snapshot (upstream commit ${new_commit}):"
        $GIT log --pretty=oneline "${old_commit}..HEAD" | $SED 's|^\(.\{8\}\)[^ ]\{1,\} |  - \1: |'
        echo
        sed -n '/^%changelog$/,$ p' "${fullspecfile}" | sed '1d'
    } > "${fullspecfile}.new"

    diff -U 5 "${fullspecfile}" "${fullspecfile}.new" || :

    echo
    echo "Adding changelog entry to ${specfile}."
    echo "Enter 'n' or 'no' to abort; anything else to continue."
    read response
    if test "x$respose" = "xn" || test "x$response" = "xno"; then 
        rm -f "${fullspecfile}.new"
    else
        mv -f "${fullspecfile}.new" "${fullspecfile}"
    fi

    cd "$pre_update_dir"
} # End of update_changelog() function


# Prepare for action
unset CDPATH
set -ex

# Update/check out source
if [ -s "${srcdir}/.git/refs/heads/master" ]; then
    ( cd "${srcdir}" && $GIT reset --hard HEAD && $GIT checkout master )
    if "$opt_skip_pull"; then
	echo "Skipping pull from upstream as requested"
    else
        # update from repo, keeping local changes separate
	( cd "${srcdir}" && $GIT fetch && $GIT rebase origin ) || exit $?
    fi
    ( cd "${srcdir}" && ($GIT branch -D "${branch}" ||:))
else
    # first checkout
    $GIT clone "git://anongit.freedesktop.org/git/xorg/driver/${tarbase}" "${srcdir}"
fi
"${do_branch}" && ( cd "${srcdir}" && $GIT checkout -b "${branch}" )

# Determine latest upstream commit:
upstream_commit="$(cd "${srcdir}" && $GIT log | head -n1 | cut -d' ' -f2)"

# This patch is waiting for the time when we actually need a radeonhd.xinf file.
#    ${name}-Generate-xinf-file-for-hwdata-from-rhd_id.c.patch

# Fix and/or clean up upstream's git repo before creating a dist tarball:
#  * upstream tends to forget to make sure the dist tarball builds
#  * we avoid ugly simultanous spec patches to Makefile.{am,in}
for patch in
do
    [ -f "$patch" ] || continue
    if "${do_branch}"; then :; else
	echo "Fatal: do_branch should be true."
	exit 3
    fi
    if [ -s "$patch" ]; then
	(cd "$srcdir" && $GIT am -3) < "$patch"
	if test -s "${srcdir}/.dotest/patch"; then
	    echo "Fatal: Could not cleanly apply patch \"$patch\"."
	    exit 11
	fi
    else
	echo "Fatal: Patch file \"$patch\" not found."
	exit 13
    fi
done

# Generate and configure build system
autoreconf -vis "${srcdir}"
mkdir "${builddir}"
( cd "${builddir}" && ../${srcdir}/configure --enable-maintainer-mode)

# Build dist tarball and move it to CWD
version=`$SED -n '/^PACKAGE_VERSION = / { s/^PACKAGE_VERSION = //;p }' "${builddir}/Makefile"`
tarname="${tarbase}-${version}-${date}git"
if "$opt_no_checks"; then
    disttarget="dist"
else
    disttarget="distcheck"
fi
make -C "${builddir}" "${disttarget}" distdir="${tarname}"
bzip2 -c -d "${builddir}/${tarname}.tar.bz2" | lzma -c > "${tarname}.tar.lzma"

# Update spec file with the proper timestamp
old_git_commit=`$SED -n 's/^%define git_commit \(.*\)/\1/p' < "$specfile"`
if "$opt_update_config"; then
    sed_update_cond "$specfile" \
	-e "s/^%define git_commit .*/%define git_commit ${upstream_commit}/" \
	-e "s/^%define date .*/%define date ${date}/" \
        -e "s/^Version:.*/Version:       ${version}/"

    sed_update_cond "${name}-README.fedora" \
	-e "s/^\(Latest upstream commit:\).*/\\1 ${upstream_commit}/"

    update_changelog "$specfile" "$old_git_commit" "$upstream_commit"
fi

# Clean up after us (keeping the patched git branch)
rm -rf "${builddir}"

echo "Old upstream commit in spec: ${old_git_commit}"
echo "Latest upstream commit:      ${upstream_commit}"
echo "$ make new-sources FILES=\"${tarname}.tar.lzma\""

